-- Object-Oriented Menu function by MikuAuahDark
-- This file is used to learn how the code works

-- Create local variable
local Menu={}
-- Create new index
Menu.list={}

-- Create function when isn't exist
New=New or function(t,...)
	-- Call the "New" index at table "t" and return the value
	return t.New(...)
end

-- Create new index. It's a function, core function
function Menu.New(title,_sel)
	-- Local table
	local temp={}
	-- Id that used to identify the menu
	local id=0
	-- Insert the empty table
	table.insert(Menu.list,temp)
	-- Let's loop to find the menu ID
	for n,v in pairs(Menu.list) do
		-- Check if the table referenced by "v" is same as "temp"
		if v==temp then
			-- It's it. Set local variable
			id=n
			-- Break to prevent more loops
			break
		end
	end
	-- Set title string
	temp.title=title
	-- Create 2 new table
	temp.selstr={}
	temp.selfunc={}
	-- Create function to show it(Call it with <MenuObject>:Show(NOT with <MenuObject>.Show)
	function temp:Show(_id)
		-- create local variable that represent the menu title
		local t=self.title..string.rep(" ",id)
		-- Do 9x loop for every selection
		for i=1,9 do
			-- Concat
			t=t..","..(self.selstr[i] or "")
		end
		-- Show menu
		menu(_id,t..","..table.concat(self.selstr,","))
	end
	-- Create function that allows you to change the selection info
	function temp:SetSel(sel,str,func)
		-- Check if it's not nil
		if str~=nil then
			-- Change
			self.selstr[sel]=str
		end
		-- Check if it's not nil
		if type(func)=="function" then
			-- Change
			self.selfunc[sel]=func
		end
	end
	-- Create function that allows you to change the title(same as <MenuObject>.title="Menu Title")
	function temp:SetTitle(lpcstr)
		-- Set value
		temp.title=lpcstr
	end
	-- Set the meta tables
	setmetatable(temp,{
		-- Handle for the negation operation(example: "Menu title is "..(-MenuObject).."" it would output "Menu title is Menu Title")
		__unm=function(self)
			-- Return value
			return self.title
		end,
		-- How does when the menu object is called? like MenuObject(id)? Then call "Show" function
		__call=function(self,_id)
			-- Call function
			self:Show(id)
		end,
		-- Tostring operation
		__tostring=function()
			-- Return "Menu (id)"
			return "Menu ("..id..")"
		end,
	})
	-- Check for the menu selection arguments
	for n,v in pairs(_sel) do
		-- Set the selection option
		temp:SetSel(n,v[1],v[2])
	end
	-- Return the object
	return temp
end

-- Create Hook
addhook("menu","Menu.Callback")
-- Create function
function Menu.Callback(id,men,sel)
	-- Do loop at menu list
	for n,v in pairs(Menu.list) do
		-- Check if the menu is same
		if ((-v)..string.rep(" ",n))==men then
			-- Use pcall so it's safe
			pcall(v.selfunc[sel],id,men,sel)
			-- Break to prevent more loops
			break
		end
	end
end

-- Set the metatable
setmetatable(Menu,{
	-- You're not allowed to edit this metatable
	__metatable=function(_)
		-- Return the "table: xxxxxxxx"(string)
		return tostring(_)
	end,
	-- You're not allowed to store a value at this table
	__newindex=function(_)
		-- Error
		error("Access denied!",2)
	end
})

-- Return the "Menu" table
return Menu